package com.huaching.k8s.service.impl;

import com.huaching.k8s.pojo.vo.PodVO;
import com.huaching.k8s.res.ResponseBO;
import com.huaching.k8s.service.KubectlService;
import com.huaching.k8s.util.DateDistanceUtil;
import io.fabric8.kubernetes.api.model.*;
import io.fabric8.kubernetes.client.KubernetesClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class KubectlServiceImpl implements KubectlService {

    @Autowired
    private KubernetesClient kubernetesClient;


    @Override
    public ResponseBO getNamespaceList() {
        return ResponseBO.Builder.init().setData(kubernetesClient.namespaces().list()).build();
    }

    @Override
    public ResponseBO getPodList(String namespace, String podName) {

        PodList podList = kubernetesClient.pods().inNamespace(namespace).list();
        List<PodVO> podVOList = new ArrayList<>();
        podList.getItems().forEach(pod -> {
            ObjectMeta metadata = pod.getMetadata();
            PodVO podVO = new PodVO();
            podVO.setNamespace(metadata.getNamespace());
            podVO.setPodName(metadata.getName());

            List<ContainerStatus> containerStatusList = pod.getStatus().getContainerStatuses();
            if (containerStatusList.size() >= 1){
                ContainerStatus containerStatus = containerStatusList.get(0);
                podVO.setReady(Optional.ofNullable(containerStatus.getReady()).orElse(false));
                podVO.setRestarts(Optional.ofNullable(containerStatus.getRestartCount()).orElse(0));
            }

            PodStatus podStatus = pod.getStatus();
            podVO.setStatus(Optional.ofNullable(podStatus.getPhase()).orElse(""));
            String startTime = Optional.ofNullable(podStatus.getStartTime().replaceAll("T"," ")
                    .replaceAll("Z","")).orElse("");
            podVO.setStartTime(startTime);
            podVO.setAge(DateDistanceUtil.getDistanceTime(startTime));
            podVOList.add(podVO);
        });

        // podName不等于空，就选出来带有podName的pod
        if (podName != null && !podName.isEmpty()){
            return ResponseBO.Builder.init().setData(podVOList.stream()
                    .filter(podVO -> podVO.getPodName().indexOf(podName) > -1).collect(Collectors.toList())).build();
        }
        return ResponseBO.Builder.init().setData(podVOList).build();
    }

    @Override
    public ResponseBO getPod(String namespace, String podName) {
        Pod pod = kubernetesClient.pods().inNamespace(namespace).withName(podName).get();
        return ResponseBO.Builder.init().setData(pod).build();
    }

    @Override
    public ResponseBO getPodLogs(String namespace, String podName) {
        String logs = kubernetesClient.pods().inNamespace(namespace).withName(podName).getLog();
        return ResponseBO.Builder.init().setData(logs).build();
    }

    @Override
    public ResponseBO deletePod(String namespace, String podName) {
        kubernetesClient.pods().inNamespace(namespace).withName(podName).delete();
        return ResponseBO.Builder.init().build();
    }

    public static void main(String[] args) {
        System.out.println("adsd".contains("asf"));
    }
}
